import csv
import os
import socket
from datetime import datetime, timedelta
import performance_test
import numpy as np

import matplotlib as mpl
mpl.use('QT5Agg')
from matplotlib import pyplot as plt
import matplotlib.patches as mplpatches


class PerformanceTHREDDS(object):

    def __init__(self):

        self.tests = []
        self.total_attempts = 100

        self.scratch_folder = '/scratch/Earth/jvegas/THREDDS_test/'
        self.results_folder = 'esnas'
        self.url_thredds = 'http://earth.bsc.es/thredds/fileServer/exp/ecmwf/system4_m1/6hourly/tas/tas_19810101.nc'
        self.data_path = '/esnas/exp/ecmwf/system4_m1/6hourly/tas/tas_19810101.nc'

        self.file_size = None
        self.node = None

        self.marker_size = 6

    @property
    def url_thredds(self):
        return self._url_thredds

    @url_thredds.setter
    def url_thredds(self, value):
        self._url_thredds = value
        self._url_fileServer = value.replace('/dodsC/', '/fileServer/')
        self._url_dodsC = value.replace('/fileServer/', '/dodsC/')

    def test_performance(self):
        print('Testing performance')
        print('THREDDS test url: {0}'.format(self._url_fileServer))
        print('Copy file path:   {0}'.format(self.data_path))

        self.tests.append(performance_test.DodscTest(self.scratch_folder, self._url_dodsC))
        self.tests.append(performance_test.NcksTest(self.scratch_folder, self.data_path))
        self.tests.append(performance_test.SubsetTest(self.scratch_folder, self._url_dodsC))
        self.tests.append(performance_test.CopyTest(self.scratch_folder, self.data_path))
        self.tests.append(performance_test.FileServerTest(self.scratch_folder, self._url_fileServer))
        self.tests.append(performance_test.DirectRetrieveTest(self.scratch_folder, self.data_path))

        self.node = socket.gethostname()
        self.file_size = os.path.getsize(self.data_path)
        self.tests_start = datetime.now()

        if not os.path.isdir(self.scratch_folder):
            os.makedirs(self.scratch_folder)

        for x in range(self.total_attempts):
            print ('Test {0} of {1}'.format(x+1, self.total_attempts))
            for test in self.tests:
                test.make_test()

        self.tests_end = datetime.now()

        print('Tests finished!')

        for test in self.tests:
            test.compute_results()
            test.clean()


    def save_results(self):
        csv_path = self._get_result_filepath('csv')
        print ('Saving result at {0}'.format(csv_path))
        with open(csv_path, 'wb') as csvfile:
            spamwriter = csv.writer(csvfile)
            spamwriter.writerow(['File', 'Filesize', 'Node'])
            spamwriter.writerow([self.data_path, self.file_size, self.node])
            line = ['Attempt']
            for test in self.tests:
                line.append(test.name)
            spamwriter.writerow(line)
            for x in range(self.total_attempts):
                line = [x]
                for test in self.tests:
                    line.append(test.time[x])
                spamwriter.writerow(line)
        print('Saving done!')

    def load_results(self, filepath):
        if not os.path.isfile(filepath):
            filepath = os.path.join(self.results_folder, filepath)
        with open(filepath, 'r') as csvfile:
            reader = csv.reader(csvfile)
            row = reader.next()
            if row[0] == 'File':
                self.data_path, self.file_size, self.node = reader.next()
                self.file_size = float(self.file_size)
                row = reader.next()
            if row[0] == 'Attempt':
                for name in row:
                    if name == performance_test.CopyTest.name:
                        self.tests.append(performance_test.CopyTest(self.scratch_folder, self.data_path))
                    elif name == performance_test.DirectRetrieveTest.name:
                        self.tests.append(performance_test.DirectRetrieveTest(self.scratch_folder, self.data_path))
                    elif name == performance_test.FileServerTest.name:
                        self.tests.append(performance_test.FileServerTest(self.scratch_folder, self._url_fileServer))
                    elif name == performance_test.DodscTest.name:
                        self.tests.append(performance_test.DodscTest(self.scratch_folder, self._url_dodsC))
                    elif name == performance_test.NcksTest.name:
                        self.tests.append(performance_test.NcksTest(self.scratch_folder, self.data_path))
                    elif name == performance_test.SubsetTest.name:
                        self.tests.append(performance_test.SubsetTest(self.scratch_folder, self._url_dodsC))

            for row in reader:
                for x in range(len(self.tests)):
                    self.tests[x].times.append(float(row[x + 1]))
            self.total_attempts = len(self.tests[0].times)

        self.tests_start = datetime.strptime(os.path.basename(filepath)[-20:-4], '%Y-%m-%d-%H:%M')
        for test in self.tests:
            test.compute_results()

        self.tests_end = self.tests_start + timedelta(seconds = np.sum([test.sum for test in self.tests]))

    def plot(self, save):
        print('Plotting results...')
        fontsize = 20
        fig, ax1 = plt.subplots(figsize=(16, 9))

        if self.file_size is None:
            self.file_size = os.path.getsize(self.data_path)

        fig.suptitle('Comparison between THREDDS and direct access performance on {0} '
                     '({1:%b %d %Y} {1:%H:%M} - {2:%H:%M}) \n'
                     'File used: {3} ({4:0.2f} GB)'.format(self.node, self.tests_start, self.tests_end, self.data_path,
                                                           self.file_size / (1024 * 1024 * 1024.0)),
                     fontsize=fontsize)

        self._attempts = np.array(range(self.total_attempts))
        for test in self.tests:
            ax1.plot(self._attempts, test.time, 'o', color=test.color, markersize=self.marker_size)
            plt.axhspan(test.average - test.std, test.average + test.std, facecolor=test.color, alpha=0.3)
            plt.axhline(y=test.average, color=test.color)

        def time_ticks(x, pos):
            d = timedelta(seconds=x)
            return str(d)

        ax1.set_ylabel('Time', fontsize=fontsize)
        ax1.set_xlabel('Attempt', fontsize=fontsize)
        ax1.set_xlim([-0.5, self.total_attempts - 0.5])

        max_time = 60
        max_ellapsed = max([np.amax(test.time) for test in self.tests])
        while max_ellapsed > max_time:
            max_time += 10
        ax1.set_ylim([0, max_time])
        formatter = mpl.ticker.FuncFormatter(time_ticks)
        ax1.yaxis.set_ticks(np.arange(0, max_time, 60))
        ax1.yaxis.set_ticks(np.arange(0, max_time, 10), minor=True)
        ax1.yaxis.set_major_formatter(formatter)
        ax1.yaxis.grid(which='major', color='black', linestyle='-')
        ax1.yaxis.grid(which='minor', color='grey', linestyle='--')

        plt.tick_params(axis='x', which='both', bottom='off', top='off', labelbottom='off')

        patches = [mplpatches.Patch(color=test.color, label=test.name)for test in self.tests]
        ax1.legend(handles=patches, loc='upper center', bbox_to_anchor=(0.5, -0.05),
                   fancybox=True, ncol=len(patches), frameon=True, fontsize=14)
        if save:
            print('Saving image...')
            fig.savefig(self._get_result_filepath('png'))
        else:
            print('Showing image...')
            plt.show()
        plt.close()
        print('Plotting finished!')

    def _get_result_filepath(self, extension):
        if not os.path.isdir(self.results_folder):
            os.makedirs(self.results_folder)
        return os.path.join(self.results_folder, 'cp_{2}_{0:%Y-%m-%d-%H:%M}.{1}'.format(self.tests_start, extension,
                                                                                        self.node))


